{% extends "error.html" %}

{% block extra_style %}
  #kernel-select {
      width: 200px;
      height: 40px;
      margin-bottom: 25px;
  }
{% endblock %}


{% block content %}
<h2>Select a valid kernel</h2>
<select name="kernel" id="kernel-select" onchange="window.location.search = '?kernel='+event.target.value">
  <option value="" disabled selected>-- Please choose an option --</option>
  {% for kernel in kernels %}
  <option value="{{ kernel }}">{{ kernel }}</option>
  {% endfor %}
</select>
{% endblock %}
